# Generated by roxygen2: do not edit by hand

S3method(print,PTXQC_table)
export(FilenameMapper)
export(LCS)
export(LCSn)
export(QCMetaFilenames)
export(RTalignmentTree)
export(YAMLClass)
export(assembleMZQC)
export(assignBlocks)
export(boxplotCompare)
export(byX)
export(byXflex)
export(correctSetSize)
export(createReport)
export(createYaml)
export(delLCP)
export(delLCS)
export(getECDF)
export(getFragmentErrors)
export(getHTMLTable)
export(getMQPARValue)
export(getMaxima)
export(getPCA)
export(getReportFilenames)
export(grepv)
export(lcpCount)
export(lcsCount)
export(longestCommonPrefix)
export(longestCommonSuffix)
export(modsToTable)
export(modsToTableByRaw)
export(mosaicize)
export(pasten)
export(pastet)
export(peakWidthOverTime)
export(plotTable)
export(plot_CalibratedMSErr)
export(plot_Charge)
export(plot_ContEVD)
export(plot_ContUser)
export(plot_ContUserScore)
export(plot_ContsPG)
export(plot_CountData)
export(plot_DataOverRT)
export(plot_IDRate)
export(plot_IDsOverRT)
export(plot_IonInjectionTimeOverRT)
export(plot_MBRAlign)
export(plot_MBRIDtransfer)
export(plot_MBRgain)
export(plot_MS2Decal)
export(plot_MS2Oversampling)
export(plot_MissedCleavages)
export(plot_RTPeakWidth)
export(plot_RatiosPG)
export(plot_ScanIDRate)
export(plot_TIC)
export(plot_TopN)
export(plot_TopNoverRT)
export(plot_UncalibratedMSErr)
export(plot_peptideMods)
export(pointsPutX)
export(qcMetric)
export(qualBestKS)
export(qualCentered)
export(qualCenteredRef)
export(qualGaussDev)
export(qualHighest)
export(qualLinThresh)
export(qualMedianDist)
export(qualUniform)
export(read.MQ)
export(renameFile)
export(repEach)
export(shortenStrings)
export(simplifyNames)
export(supCount)
export(theme_blank)
exportClasses(FilenameMapper)
exportClasses(YAMLClass)
exportClasses(qcMetric)
import(R6)
import(R6P)
import(RColorBrewer)
import(UpSetR)
import(data.table)
import(ggdendro)
import(ggplot2)
import(grDevices)
import(grid)
import(gridExtra)
import(gtable)
import(htmlTable)
import(knitr)
import(magrittr)
import(methods)
import(plyr)
import(reshape2, except = c(dcast, melt))
import(rmarkdown)
import(rmzqc)
import(stats)
import(utils)
import(xml2)
import(yaml)
importFrom(seqinr,circle)
