% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fcn_misc.R
\name{RSD}
\alias{RSD}
\title{Relative standard deviation (RSD)}
\usage{
RSD(x)
}
\arguments{
\item{x}{Vector of numeric values}
}
\value{
RSD
}
\description{
Simply \code{\link{CV}}*100
}
