% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fcn_misc.R
\name{del0}
\alias{del0}
\title{Replace 0 with NA in a vector}
\usage{
del0(x)
}
\arguments{
\item{x}{A numeric vector}
}
\value{
Vector of same size as 'x', with 0's replaced by NA
}
\description{
Replace 0 with NA in a vector
}
