% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fcn_miscGGplot.R
\name{theme_blank}
\alias{theme_blank}
\title{A blank theme (similar to the deprecated theme_blank())}
\usage{
theme_blank()
}
\value{
A ggplot2 object, representing an empty theme
}
\description{
A blank theme (similar to the deprecated theme_blank())
}
