% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PTXQC-package.R
\name{PTXQC}
\alias{PTXQC}
\title{Quality control for proteomics data}
\description{
Quality control for proteomics data
}
