% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fcn_misc.R
\name{CV}
\alias{CV}
\title{Coefficient of variation (CV)}
\usage{
CV(x)
}
\arguments{
\item{x}{Vector of numeric values}
}
\value{
CV
}
\description{
Computes sd(x) / mean(x)
}
